class MSBMouseOverlay extends MSBListButton;

// Trace Stuff - used for selecting Objects with the Mouse Pointer
// needs updating to give a List of selected objects instead of only one
var vector MouseLast3D,                     //Position of Mouse on click before
           Mouse3D;                         //actual 3D Position of the Mouse

var float clickedLocationX;                 //Location of the last click
var float clickedLocationY;

var float TraceLength;                      //when clicking, we do a Trace, this is the Trace length
var bool bHitActors;                        //used to determine if we should trace through Actors (false for moving stuff around, true for selecting Actors)
var vector hitNorm,                         //not used, but necessary
           traceExt,                        //radius of the trace
           traceEnd;                        //End Point for the Trace - determines direction

var vector xAxis,yAxis,zAxis;               //the Orientation of the Controller

//Mouse Pressing stuff and current state of Mouse
var bool btnOneDown,
         btnTwoDown,
         btnOne,                            //Left Mouse Button pressed
         btnTwo,                            //right Mouse Button pressed
         btnThree,                          //Middle Mouse Button pressed (not implemented yet)
         bShiftDown,                        //Shift pressed down (for selecting)
         bShiftRelease,                     //Shift released again
         bDeleteDown,                       //pressed delete
         bEsc,                              //pressed Esc
         bTyping;                           //we are currently typing

var int joyXaxis,                           //direct Joystick properties - not used anymore
        joyYaxis,
        joyUAxis,
        joyVaxis;

var Actor lastSelectedActor,
          worldMSBTargetActor,              //Actor the Mouse is hovering over at the moment
          hudTargetActor;                   //Button the Mouse is hovering over at the moment

var array<Actor> selectedActor;             //list of selected Actors (used for Moving/scaling, etc)
var array<MSBGUIOverlay> selectedButtons;   //list of selected Buttons

var int pressedKey;                         //currently pressed Key
var int releasedKey;                        //currently released Key

var string TextInput;                       //Text written

var string ButtonPrefs;              //displayed in PreferencesWindow

var Material CursorMat;                     //see Pointers in MSBController !?

var float amp,safezone;                     //for fancy buttons


event PostBeginPlay()
{
super.PostBeginPlay();
Mouse=self;
}

//*****************************************************************
//* everything is done in the Render function                     *
//* here we determine what to do, which button is pressed, etc... *
//*****************************************************************

function Render(Canvas C)
{
local int i;

if (!bTyping)
  {
  if (hudTargetActor==None && btnOne || bEsc)
    deSelectAll();


    get3DActor();
    if(!bContained)
      drawMouse(C);
    deleteActors(C);

  // Actor functionality and Info-Text
  if (hudTargetActor==None && worldMSBTargetActor!=None && !worldMSBTargetActor.bStatic)
    {
    if(!bContained)
      DrawInfo(C);
    if (btnOne)
      LeftBtn(C);
    if (btnTwo)
      RightBtn(C);
    }

  if (hudTargetActor==None && worldMSBTargetActor==None)
    {
    hidePointers();
    if (btnOne)
      LeftBtn(C);
    if (btnTwo)
      RightBtn(C);
    }
  }
else
 {
 drawCursor(C);
 }
// Reset MouseButtons after all the Buttons have checked for Clicks
btnOne=false;
btnTwo=false;
bDeleteDown=false;
bShiftRelease=false;
bEsc=false;
releasedKey=9999;

//fancy Button positioning
if (amp<80)
  amp+=5;
for (i=0;i<ListButton.length;i++)
  if (ListButton[i]!=None)
    positionButton(C,i);
}

// *******************************************
// we pressed the left Mouse button.
// we select an Actor if the Mouse is hovering over one
// we also check if we pressed shift for multi-select
// Note: Button Menus and movement is handled in the Buttons themselves!

function LeftBtn(Canvas C)
{
//The controller is doing things when we press the left Mousebtn without hovering on a button
M.pressedLeft();

clickedLocationX=xPos;
clickedLocationY=yPos;
MouseLast3D=Mouse3D;
lastSelectedActor=worldMSBTargetActor;
}

//*************************************************
// we pressed the right button
// we display a menu taken from the buttonlist of the actor we are hovering over

function RightBtn(Canvas C)
{
local int i;
local bool bInSelection;

for (i=0;i<selectedActor.length;i++)
  if (worldMSBTargetActor==selectedActor[i])
    bInSelection=true;

if (!bInSelection)
  {
  deSelectAll();
  LeftBtn(C);
  }

ButtonList.Remove(0,ButtonList.length);
ListButton.Remove(0,ListButton.length);
fillButtonList();
super.LeftBtn(C);
assignProperties();
deselect();
clickedLocationX=xPos;
clickedLocationY=yPos;
MouseLast3D=Mouse3D;
lastSelectedActor=worldMSBTargetActor;
}

// ****************************************************************************
// generate actor MSBPropertyButtons (stuff that appears when clicking on actor)
// and also generate Context Menu (stuff that appears when clicking on nothing)
// ****************************************************************************
function fillButtonList()
{
local MSBActor MSA;
local MSBPawn MSP;
local MSBLight MSL;
local int i;

MSA=MSBActor(worldMSBTargetActor);
if (MSA!=None)
  {
  for (i=0;i<MSA.Buttonlist.length;i++)
    Buttonlist[i]=MSA.ButtonList[i];
  }
else
  {
  MSP=MSBPawn(worldMSBTargetActor);
  if (MSP!=None)
    {
    for (i=0;i<MSP.Buttonlist.length;i++)
      Buttonlist[i]=MSP.ButtonList[i];
    }
  else
    {
    MSL=MSBLight(worldMSBTargetActor);
    if (MSL!=None)
      {
      for (i=0;i<MSL.Buttonlist.length;i++)
        Buttonlist[i]=MSL.ButtonList[i];
      }
    else
      {
      M.pressedRight();
      clickedLocationX=xPos;
      clickedLocationY=yPos;
      }
    }
  }
}

// ****************************************************************************
// Assign actor MSBPropertyButtons (stuff that appears when clicking on actor)
// ****************************************************************************
function assignProperties()
{
local MSBActor MSA;
local MSBPawn MSP;
local MSBLight MSL;
local Material BtnMat;
local int i;

if(worldMSBTargetActor.isA('MSBActor'))
  MSA=MSBActor(worldMSBTargetActor);
if (MSA!=None)
  {
  for (i=0;i<MSA.Buttonlist.length;i++)
    {
      MSBPropertyButton(ListButton[i]).Property=MSA.Properties[i];
      MSBPropertyButton(ListButton[i]).Parent=worldMSBTargetActor;
      if (MSA.ButtonMat[i]!="")
        {
        MSBPropertyButton(ListButton[i]).bDisplayName=false;
        BtnMat=Material(DynamicLoadObject(MSA.ButtonMat[i],class'Material'));
        ListButton[i].Mat=BtnMat;
        ListButton[i].widthRel=48;
        ListButton[i].heightRel=48;
        ListButton[i].updateMaterial();
        }
    }
  }

if(worldMSBTargetActor.isA('MSBPawn'))
  MSP=MSBPawn(worldMSBTargetActor);
if (MSP!=None)
  {
  for (i=0;i<MSP.Buttonlist.length;i++)
    {
    if (MSP.Buttonlist[i]!="")
      {
      MSBPropertyButton(ListButton[i]).Property=MSP.Properties[i];
      MSBPropertyButton(ListButton[i]).Parent=worldMSBTargetActor;
      if (Caps(MSP.ButtonMat[i])!="MOVIESANDBOXTEXTURES.ICON_BASE")
         MSBPropertyButton(ListButton[i]).bDisplayName=false;
      if (MSP.ButtonMat[i]!="")
        {
        BtnMat=Material(DynamicLoadObject(MSP.ButtonMat[i],class'Material'));
        MSBPropertyButton(ListButton[i]).Mat=BtnMat;
        MSBPropertyButton(ListButton[i]).widthRel=48;
        MSBPropertyButton(ListButton[i]).heightRel=48;
        MSBPropertyButton(ListButton[i]).updateMaterial();
        }
      }
    }
  }

if(worldMSBTargetActor.isA('MSBLight'))
  MSL=MSBLight(worldMSBTargetActor);
if (MSL!=None)
  {
  for (i=0;i<MSL.Buttonlist.length;i++)
    {
    if (MSA.Buttonlist[i]!="")
      {
      MSBPropertyButton(ListButton[i]).Property=MSL.Properties[i];
      MSBPropertyButton(ListButton[i]).Parent=worldMSBTargetActor;
      }
    }
  }
if(MSL==None && MSP==None && MSA==None)
  {
  for (i=0;i<Buttonlist.length;i++)
    {
    if (M.ContextButtonlist[i]!="")
      {
      MSBPropertyButton(ListButton[i]).Property=M.ContextButtonCommand[i];
      MSBPropertyButton(ListButton[i]).Parent=self;
      MSBPropertyButton(ListButton[i]).BtnName=M.ContextButtonName[i];
      Log(ListButton[i]);
      }
    }
  }
}

// ****************************************************************************
// draw the Mouse Icon
// ****************************************************************************
function drawMouse(Canvas C)
{
if (xpos>C.ClipX)
  xpos=C.ClipX-10;
if (xpos<0)
  xpos=0;

if (ypos>C.ClipY)
  ypos=C.ClipY-10;
if (ypos<0)
  ypos=0;

C.SetDrawColor(255,255,255,255);
C.SetPos(xpos,ypos);
C.DrawTile(Mat,16,16,0,0,32,32);
}

function DrawCursor(Canvas C)
{
if (hudTargetActor!=None)
  {
  C.SetDrawColor(255,255,255,64);
  C.SetPos(MSBGuiOverlay(hudTargetActor).xpos+MSBGuiOverlay(hudTargetActor).width,MSBGuiOverlay(hudTargetActor).ypos);
  //mouse should not move!!
  C.DrawTile(CursorMat,64,16,0,0,32,32);
  }
}

// ****************************************************************************
//3D grabbing
// ****************************************************************************
function get3DActor()
{
local float sY,sZ, fX,fY;
local vector sVec;

if (M!=None)
  {
  M.GetAxes(M.Rotation,xAxis,yAxis,zAxis);

  //int variables don't divide well
  fX=M.screenX;
  fY=M.screenY;

  //  map ScreenX between -1 and 1
  sY= (xPos/fX)*2 -1;
  sZ= (yPos/fY) * fY/fX *2 - fY/fX;

  // calculate screenvector
  sVec.Y=sY;
  sVec.X=1;   //distance from eyes to plane is believed 1
  sVec.Z=-sZ; //flip MouseY
  sVec=normal(sVec) * TraceLength;

  traceEnd=sVec;
  //transform with view Coords
  traceEnd=sVec.x * xAxis + sVec.y * yAxis + svec.Z*zAxis;

  //incorporate Player Location
  traceEnd=traceEnd+M.Location;

  //updating worldTargetActor
  worldMSBTargetActor=Trace(Mouse3D,hitnorm,traceEnd,M.Location,bHitActors,traceExt);

  if (worldMSBTargetActor!=None && ( worldMSBTargetActor.IsA('LevelInfo') || worldMSBTargetActor.IsA('TerrainInfo') || worldMSBTargetActor.bStatic) )
    worldMSBTargetActor=None;
//special handling for wrapped actors!
  if (worldMSBTargetActor!=None && worldMSBTargetActor.Tag=='Wrapped')
    worldMSBTargetActor=WorldMSBTargetActor.Owner;
  }
}



// ****************************************************************************
// draw Actor info - draws the current Actor Name and Location
// ****************************************************************************
function DrawInfo(Canvas C)
{
local int i;
  C.DrawText(worldMSBTargetActor.Location.X);
  C.SetPos(xpos,ypos+10);
  C.DrawText(worldMSBTargetActor.Location.Y);
  C.SetPos(xpos,ypos+20);
  C.DrawText(worldMSBTargetActor.Location.Z);
  C.SetPos(xpos,ypos+30);
  C.DrawText(worldMSBTargetActor.GetItemName(String(worldMSBTargetActor)));
if (worldMSBTargetActor.IsA('MSBActor'))
  {
  for (i=0;i<MSBActor(worldMSBTargetActor).Keys.length;i++)
    {
    if (M.pointers[i]==None)
      M.pointers[i]=Spawn(class'MSBPointActor');
    M.pointers[i].SetLocation(MSBActor(worldMSBTargetActor).Keys[i].Location);
    C.SetPos(xpos,ypos+40);
    C.DrawText(MSBActor(worldMSBTargetActor).Keys[i].keyNumber);
    M.pointers[i].bHidden=false;
    }
  }
}

// ****************************************************************************
// deselect Actors and Buttons
// ****************************************************************************
function deSelectAll()
{
local int i;
//clicking in the Open deSelects ListButtons and Stuff

for (i=0;i<M.myHUD.Overlays.length;i++)
  {
  MSBGuiOverlay(M.myHUD.Overlays[i]).deSelect();
  if (MSBGuiOverlay(M.myHUD.Overlays[i]).SubLevel>0)
    M.myHUD.Overlays[i--].Destroy();
  }
if (!bShiftDown || worldMSBTargetActor==None)
  {
  selectedActor.Remove(0,selectedActor.length);
  hideSelectPointers();
  }
selectedButtons.Remove(0,selectedButtons.length);
amp=0;
}

// ****************************************************************************
// Delete Actors and Buttons
// ****************************************************************************
function deleteActors(Canvas C)
{
local int i;

if (bDeleteDown)
  {

// deleting Objects from the Hud

  if (hudTargetActor!=None && hudTargetActor.IsA('MSBNodeButton'))
    {
    if (MSBNodeButton(hudTargetActor).Order.IsA('MSBRootNode'))
      {
      worldMSBTargetActor=MSBRootNode(MSBNodeButton(hudTargetActor).Order).BasePawn;
      clearPawn();
      worldMSBTargetActor=None;
      }
    else
      {
      MSBGuiOverlay(hudTargetActor).deSelect();
      if (MSBNodeButton(hudTargetActor).NodeIn!=None)
        {
        MSBNodeButton(hudTargetActor).NodeIn.ClearLinks();
        MSBNodeButton(hudTargetActor).NodeIn.Destroy();
        }
      if (MSBNodeButton(hudTargetActor).NodeOut!=None)
        {
        MSBNodeButton(hudTargetActor).NodeOut.ClearLinks();
        MSBNodeButton(hudTargetActor).NodeOut.Destroy();
        }
      MSBNodeButton(hudTargetActor).Order.Destroy();
      for (i=0;i<M.LevelNodes.length;i++)
        if (M.LevelNodes[i]==hudTargetActor)
          M.LevelNodes.Remove(i,1);
      hudTargetActor.Destroy();
      }
    }

  if (hudTargetActor!=None && hudTargetActor.IsA('MSBScriptNodeButton'))
    {
    MSBGuiOverlay(hudTargetActor).deSelect();
    if (MSBScriptNodeButton(hudTargetActor).NodeOut!=None)
      {
      MSBScriptNodeButton(hudTargetActor).NodeOut.ClearLinks();
      MSBScriptNodeButton(hudTargetActor).NodeOut.Destroy();
      }
     for (i=0;i<M.LevelMSBScriptNodes.length;i++)
       if (M.LevelMSBScriptNodes[i]==hudTargetActor)
         M.LevelMSBScriptNodes.Remove(i,1);
     hudTargetActor.Destroy();
     }

  if (hudTargetActor!=None && hudTargetActor.IsA('MSBContainerButton'))
    {
    MSBContainerButton(hudTargetActor).hidecontent(false);
    for (i=0;i<M.LevelContainers.length;i++)
      if (M.LevelContainers[i]==hudTargetActor)
        M.LevelContainers.Remove(i,1);
    hudTargetActor.Destroy();
    return;
    //todo: fill in Code for deleting containers
    }

//don't do anything in case we want to delete NodeIn or NodeOut only
  if (hudTargetActor!=None && hudTargetActor.IsA('MSBNodeInButton') || hudTargetActor.IsA('MSBNodeOutButton'))
    {
    return;
    }

//delete Savegame Names
  if(hudTargetActor!=None && hudTargetActor.IsA('MSBPropertyButton') && MSBPropertyButton(hudTargetActor).Parent.IsA('MSBSceneSaveButton'))
    {
    Log("deleting a savegame");
    for(i=0;i<M.SaveGames.length;i++)
     if (M.SaveGames[i]==MSBGuiOverlay(hudTargetActor).BtnName)
       {
       M.SaveGames.Remove(i,1);
       M.SaveGeneralConfig();
       }
    }

//killing everything else
  if (hudTargetActor!=None && hudTargetActor.IsA('MSBGuiOverlay') && !MSBGuiOverlay(hudTargetActor).bMainMenu)
    {
    for (i=0;i<M.LevelNodes.length;i++)
      if (M.LevelNodes[i]==hudTargetActor)
        M.LevelNodes.Remove(i,1);
    hudTargetActor.Destroy();
    }

// deleting Objects from the Scene

  if (worldMSBTargetActor!=None && worldMSBTargetActor.IsA('MSBActor'))
    clearActor();
  else if (worldMSBTargetActor!=None && worldMSBTargetActor.IsA('MSBPawn'))
    clearPawn();
  else if (worldMSBTargetActor!=None && worldMSBTargetActor.IsA('Actor'))
    {
    for (i=0;i<M.LevelSpecial.length;i++)
      if (M.LevelSpecial[i]==worldMSBTargetActor)
        M.LevelSpecial.Remove(i,1);
    worldMSBTargetActor.Destroy();
    Log("Destroyed an Actor");
    }
  else                                 // don't destroy things we don't understand
     Log("I don't know this type of thing to remove:"@hudtargetActor@worldMSBTargetActor);
//    worldMSBTargetActor.Destroy();
  deSelectAll();
  }
}


//---------------------------------------------------------------------
// cleaning up Actors, KeyFrames and Pawns after Deletion
//---------------------------------------------------------------------

function clearKeyFrame()
{
local MSBActor KeyOwner;
local MSBKeyFrame Key;
local int i;

Key=MSBKeyFrame(worldMSBTargetActor);
KeyOwner=MSBActor(Key.Owner);

//remove Key
KeyOwner.Keys.Remove(Key.KeyNumber,1);

//reassign KeyNumbers
for(i=0;i<KeyOwner.keys.length;i++)
  KeyOwner.Keys[i].KeyNumber=i;

Key.Destroy();
Log("Destroyed an MSBKeyFrame!");
}

function clearPawn()
{
local string ObjectName, MapName;
local int j,i;
local MSBNodeButton NB;

for (j=0;j<M.LevelNodes.length;j++)
  {
  NB=M.LevelNodes[j];
  for (i=0;i<NB.ButtonList.length;i++)
   {
    if (Caps(NB.ButtonList[i])=="MSBPROPERTYBUTTONS.MSBPROPERTYBUTTON3D")
      {
      ObjectName=NB.Order.GetPropertyText(NB.Properties[i]);
      ObjectName=GetItemName(ObjectName);
      ObjectName-="'";
      MapName=Left(string(Level), InStr(string(Level), "."));
      ObjectName=MapName$"."$ObjectName;
      if (Actor(FindObject(ObjectName,class'Actor'))==worldMSBTargetActor)
        NB.Order.SetPropertyText(NB.Properties[i],"None");
      }
    }
   if (NB.Order.IsA('MSBRootNode') && MSBRootNode(NB.Order).BasePawn==worldMSBTargetActor)
    {
    NB.deSelect();
    if (NB.NodeIn!=None)
      {
      NB.NodeIn.ClearLinks();
      NB.NodeIn.Destroy();
      }
    if (NB.NodeOut!=None)
      {
      NB.NodeOut.ClearLinks();
      NB.NodeOut.Destroy();
      }
    NB.Order.Destroy();
    for (i=0;i<M.LevelNodes.length;i++)
      if (M.LevelNodes[i]==hudTargetActor)
        M.LevelNodes.Remove(i,1);
    NB.Destroy();
    }
  }
for (i=0;i<M.LevelPawns.length;i++)
  if (M.LevelPawns[i]==worldMSBTargetActor)
    {
    M.LevelPawns[i].clearPawn();
    M.LevelPawns.Remove(i,1);
    }
worldMSBTargetActor.Destroy();
Log("Destroyed an MSBPawn!");
}

function clearActor()
{
local string ObjectName, MapName;
local int j,i;
local MSBNodeButton NB;



//remove from all Node References
for (j=0;j<M.LevelNodes.length;j++)
  {
  NB=M.LevelNodes[j];
  for (i=0;i<NB.ButtonList.length;i++)
   {
    if (Caps(NB.ButtonList[i])=="MSBPROPERTYBUTTONS.MSBPROPERTYBUTTON3D")
      {
      ObjectName=NB.Order.GetPropertyText(NB.Properties[i]);
      ObjectName=GetItemName(ObjectName);
      ObjectName-="'";
      MapName=Left(string(Level), InStr(string(Level), "."));
      ObjectName=MapName$"."$ObjectName;
      if (Actor(FindObject(ObjectName,class'Actor'))==worldMSBTargetActor)
        NB.Order.SetPropertyText(NB.Properties[i],"None");
      }
    }
  }

//remove from Controller List
for (i=0;i<M.LevelActors.length;i++)
  if (M.LevelActors[i]==worldMSBTargetActor)
    {
    M.LevelActors[i].clearActor();
    M.LevelActors.Remove(i,1);
    }

//special handling for removing a single keyframe
if (worldMSBTargetActor.IsA('MSBKeyFrame'))
    clearKeyFrame();
else
  {
  //ToDo: clear all KeyFrames...
  if (MSBActor(worldMSBTargetActor).Keys.Length>0)
    for (i=0;i<MSBActor(worldMSBTargetActor).Keys.Length;i++)
      MSBActor(worldMSBTargetActor).Keys[i].Destroy();
  worldMSBTargetActor.Destroy();
  Log("Destroyed an MSBActor!");
  }
}

function hidePointers()
{
local int i;

if (!M.pointers[0].bHidden)
  for (i=0;i<M.pointers.length;i++)
    M.pointers[i].bHidden=true;
}

function hideSelectPointers()
{
local int i;

Log("hiding SelectPointers..."@M.SelectPointers[0].bHidden@M.SelectPointers.length);

if (!M.SelectPointers[0].bHidden)
  for (i=0;i<M.SelectPointers.length;i++)
    M.SelectPointers[i].bHidden=true;
}

//************************************************************************
//* Right Click Menu Shape
//************************************************************************

function positionButton(Canvas C,int i, optional float x, optional float y)
{
local float num,freq,ampX,ampY,circNum;
local float sideAmount,lr,ud;
local bool bLeftRight;

//Reset variables
ampX=amp*C.ClipX/1024;   //amplitude of circle
ampY=amp*C.ClipX/1024;
num=i+1;
sideAmount=4;           //how many quadrants do we use? Used later in freq
lr=1;                   //left and right side
ud=1;                   //up and down side

circNum=FClamp(ListButton.Length/4,1,5);  //determine number of circles by dividing the total number of buttons to draw by 4


//determine where we can draw
if (clickedLocationX<safezone)
  {
  sideAmount=2;
  lr=-1;
  ampY*=1.5;
  bLeftRight=true;
  }
if (C.ClipX-clickedLocationX<safezone)
  {
  sideAmount=2;
  lr=1;
  ampY*=1.5;
  bLeftRight=true;
  }
if (clickedLocationY<safezone)
  {
  if (sideAmount<4)
    sideAmount-=1;
  else
    sideAmount=2;
  ud=-1;
  ampX*=1.2;
  }
if (C.ClipY-clickedLocationY<safezone)
  {
  if (sideAmount<4)
    sideAmount-=1;
  else
    sideAmount=2;
  ud=1;
  ampX*=1.2;
  }

//draw the appropriate amount of icons

//draw a full, half, or quarter circle
freq=sideAmount*0.5;

//special case for left and right border
if (sideAmount==2 && bLeftRight)
  {
  ListButton[i].xPos=clickedLocationX-lr*ampX+sin(num/(ListButton.length+1)*freq*Pi-(lr+ud)*0.5*Pi)*(sublevel+1)*ampX*((num%circNum)/(circNum*0.75)+1);
  ListButton[i].yPos=clickedLocationY-cos(num/(ListButton.length+1)*freq*Pi-(lr+ud)*0.5*Pi)*(sublevel+1)*ampY*((num%circNum)/(circNum*0.75)+1);
  }

//general drawing of Icons. Depending on frequency and starting position as determined above
else
  {
  ListButton[i].xPos=clickedLocationX+sin(num/(ListButton.length+1)*freq*Pi-lr*0.5*Pi)*(sublevel+1)*((num%circNum)+1)*ampX;
  ListButton[i].yPos=clickedLocationY-cos(num/(ListButton.length+1)*freq*Pi-ud*0.5*Pi)*(sublevel+1)*((num%circNum)+1)*ampY;
  }
//((num%circNum)+1) for amount of circles
//merken fr DOFOVerlay!!!
//Special Handling for Buttons that will destroy screen setup (e.g. PropertyButtonViewPort)
if (MSBPropertyButton(ListButton[i]).bRenderToTextured)
  {
  ListButton[i].xPos=C.ClipX-(ListButton[i].width+4);
  ListButton[i].yPos=C.ClipY-(ListButton[i].height+4);
  }
}

defaultproperties
{
Mat=Texture'MoviesandboxTextures.Icons.MouseOverlay'
CursorMat=Texture'MoviesandboxTextures.icon_node'

TraceLength=5000
traceExt=(X=1,Y=1,Z=1)
bHitActors=true
widthRel=32
heightRel=32
safezone=150
maxListSize=20
}
