// ***************************************************************************************
// Here's how Depth of field works:
//
// We simulate a lens by rendering the view from certain points next to the actual Viewport
// looking at the same target (the focal point) and then superimposing those renderings
// on the screen.
// this implementation uses a "+" arrangement of 4 rendertargets. You might want to change that to an "x" orientation
//
// note: ScriptedTexture[0] (the middle viewtarget) never gets displayed for algorithmic reasons and coz i was lazy.
//
// Limitations:
// seems to have issues in levels without skyZone and with some Staticmeshes
// ***************************************************************************************

class MSBDepthOfFieldOverlay extends HudOverlay
config (MSBCode);

var ColorModifier Col[90];        //used for alpha blending of the scriptedtextures
var ScriptedTexture Tex[90];      //Texture to render screen to. 90 is the max number of rendertargets

//Lens
var bool bLens;
var Material LensTexture;
var Shader Shad;

var MSBCamTexClientDoF ctc[90];               //this is an actor with the scriptedtexture on,
                                  //for some reason, an actor with the scriptedtexture has to be present and "visible" or it fucks up rendering
                                  //also serves as the viewpoint for scriptedtextures

var() Actor Target;               //the focus point
var () config int maxRenders;               //number of rendertargets
var() config int resX,resY;                //DoF Scriptedtexture resolution
var int ucl, vcl;

var() config int FOV,Alpha;

var() bool bReady,bResetP;        //internal stuff for the MSB

var() config float fuzzyness, flickr;            //scriptedtexture displacement

var MSBCamera MSC;       //can be any actor. Is special for my implementation

// ***************************************************************************************
// create the hackactors. Also create the textures and initialise setup
// ***************************************************************************************

function iniOverlays()
{
local int i;

Log("initializing Overlays");

for (i=0;i<maxRenders;i++)
  {
  ctc[i]=spawn(class'MSBCamTexClientDoF',self);
  ctc[i].CameraActor=MSC;
  if (Tex[i]==None)
    Tex[i]=new class'ScriptedTexture';
  Tex[i].SetSize(resX,resY);
  Tex[i].Client=ctc[i];

  ctc[i].Skins[0]=tex[i];
  ctc[i].Tex=Tex[i];
  ctc[i].FOV=FOV;

  if (Col[i]==None)
    Col[i]=new class'ColorModifier';
  Col[i].Material=Tex[i];
  Col[i].Color.A=Alpha;
  }
//set Lens

if (MSC.bLens)
  {
  for (i=0;i<maxRenders;i++)
    {
    LensTexture=Material(DynamicLoadObject(MSC.LensMaterial, class'Material'));
    shad=new class'Shader';
    Tex[i].SetSize(resX/4,resY/4);
    shad.Diffuse=Tex[i];
    shad.Opacity=LensTexture;
    shad.Specular=LensTexture;
//    shad.OutputBlending=OB_Translucent;
//    shad.ModulateSpecular2X=true;
    Col[i].Material=Shad;
    Col[i].Color.A=Alpha+64;
    maxRenders=2;
    }
  }
target=MSC.ViewTarget;
bReady=true;
}

// ***************************************************************************************
// clean up
// ***************************************************************************************
function disableOverlays()
{
local int i;

for (i=0;i<maxRenders;i++)
  {
  Tex[i].Client=None;
  ctc[i].destroy();
  }
bReady=false;
}

// ***************************************************************************************
// change stuff at runtime. Mainly used for debugging and testing with the "editactor=classname" console command
// ***************************************************************************************
function Reset()
{
//MSC.FOV=FOV;
//MSC.PC.SetFOV(FOV);
disableOverlays();
iniOverlays();
}

// ***************************************************************************************
// Render the Scriptedtextures ontop of the frame and update
// ***************************************************************************************
simulated function Render(Canvas C)
{
local int i;
if (bReady )
  {
  C.SetDrawColor(255,255,255,255);
  for (i=0;i<maxRenders;i++)
    {
    C.SetPos(0,0);
    C.DrawTile(Col[i],C.ClipX,C.ClipY,0,0,Tex[i].USize,Tex[i].VSize);//C.ClipX*res,C.ClipY*res);
    ctc[i].CameraActor=MSC;
    }
  updateDoFCams();
  }
if (bResetP)                //used for changing FOV and Alpha(transparency) at runtime
  Reset();
}


// ***************************************************************************************
// a number of actors are arranged in a "o" pose around the camera target
// location of those actors are updated every rendercycle.
// view is rendered from the position of those actors and then superimposed on the screen
//
// ***************************************************************************************
function updateDofCams()
{
local int i;
local vector xAxis, yAxis, zAxis;

MSC.GetAxes(MSC.Rotation,xAxis,yAxis,zAxis);
Target=MSC.ViewTarget;


for(i=0;i<maxRenders;i++)
  {
  ctc[i].rellocation=0.5*zAxis*sin((i+1)/(maxRenders+1)*Pi)*((fuzzyness/3) +1);
  ctc[i].rellocation+=0.5*yAxis*cos((i+1)/(maxRenders+1)*Pi)*((fuzzyness/3) +1);

  ctc[i].SetRotation(rotator(Target.Location-ctc[i].Location));
  if (MSC.FOV>0)
    FOV=MSC.FOV;
  ctc[i].FOV=FOV;

  if (FRand()>=flickr)
    ctc[i].Tex.Revision++;
  }
}

defaultproperties
{
//flickr=0.1
resX=640
resY=480
maxRenders=6
FOV=90
Alpha=128
fuzzyness=9
}
